@@ -8,8 +8,9 @@ from discord.ext.commands import Bot as B, AutoShardedBot as AB
 from .models import Bot, SimpleUser, BotStats
 from .errors import ToppyError, Forbidden, TopGGServerError, Ratelimited, NotFound
 from json import dumps
+from discord.ext.tasks import loop
 
-__version__ = "0.1.3"
+__version__ = "1.0.0"
 _base_ = "https://top.gg/api"
 
 
@@ -26,16 +27,15 @@ class TopGG:
             bot: Union[C, B, AC, AB],
             *,
             token: str,
-            autopost_every: int = 60
+            autopost_every: int = 60,
+            autopost: bool = True,
+            # account_for_weekend: bool = True
     ):
         self.bot = bot
         self.token = token
+        # self.respect_weekend = account_for_weekend
         # noinspection PyTypeChecker
         self.session = None  # type: aiohttp.ClientSession
-        self.autopost_interval = autopost_every
-        self.autopost_task = None
-        if self.autopost_interval:
-            self.autopost_task = self.bot.loop.create_task(self._autopost())
 
         async def set_session():
             self.session = aiohttp.ClientSession(
@@ -49,22 +49,29 @@ class TopGG:
             # Why is this here?
             # There's an undesirable warning if you make a session from outside an async function. Kinda crap but meh.
         self._session_setter = self.bot.loop.create_task(set_session())
+        if autopost:
+            self.autopost.start()
+
+        # Function aliases
+        self.vote_check = self.upvote_check
+        self.has_upvoted = self.upvote_check
+        self.get_user_vote = self.upvote_check
 
     def __del__(self):
-        if self.autopost_task:
-            self.autopost_task.cancel()
+        self.autopost.stop()
 
     async def _wf_s(self):
         if not self.session:
             await self._session_setter
         return
 
-    async def _autopost(self):
-        while True:
-            await self.post_stats()
-            await asyncio.sleep(self.autopost_interval)
+    @loop(minutes=30)
+    async def autopost(self):
+        await self.post_stats()
 
     async def _request(self, method: str, uri: str, **kwargs):
+        if kwargs.get("data") and isinstance(kwargs["data"], dict):
+            kwargs["data"] = dumps(kwargs["data"])
         fail_if_timeout = kwargs.pop("fail_if_timeout", True)
         expected_codes = kwargs.pop("expected_codes", [200])
         url = _base_ + uri
@@ -91,8 +98,11 @@ class TopGG:
 
             data = await response.json()
             # inject metadata
-            data["_toppy_meta"]["headers"] = response.headers
-            data["_toppy_meta"]["status"] = response.status
+            if isinstance(data, dict):
+                data["_toppy_meta"] = {
+                    "headers": response.headers,
+                    "status": response.status
+                }
         return data
 
     async def fetch_bot(self, bot_id: int, *, fail_if_ratelimited: bool = True) -> Bot:
@@ -107,7 +117,7 @@ class TopGG:
         response["state"] = self.bot
         return Bot(**response)
 
-    async def fetch_bots(self, limit: int = 50, search: dict = None, sort: str = None,
+    async def fetch_bots(self, limit: int = 50, offset: int = 0, search: dict = None, sort: str = None,
                          fail_if_ratelimited: bool = True) -> dict:
         limit = max(2, min(500, limit))
         uri = "/bots?limit="+str(limit)
@@ -116,6 +126,8 @@ class TopGG:
             uri += "&search="+search
         if sort:
             uri += "&sort="+sort
+        if offset:
+            uri += "&offset="+str(offset)
         result = await self._request("GET", "/bots", fail_if_timeout=fail_if_ratelimited)
         new_results = []
         for bot in result["results"]:
@@ -124,6 +136,26 @@ class TopGG:
         result["results"] = new_results
         return result
 
+    async def bulk_fetch_bots(self, limit: int = 500, *args) -> dict:
+        """Similar to fetch_bots, except allows for requesting more than 500.
+
+        This is equivalent to:
+        ```python
+        batch_one = await TopGG.fetch_bots(500)
+        batch_two = await TopGG.fetch_bots(500, offset=500)
+        batch_three = await TopGG.fetch_bots(500, offset=1000)
+        ...
+        ```
+        """
+        results = {}
+        remaining = limit
+        for i in range(0, limit, 500):
+            amount = min(500, remaining)
+            batch_results = await self.fetch_bots(amount, offset=i, *args)
+            remaining -= amount
+            results = {**results, **batch_results}
+        return results
+
     async def fetch_votes(self, *, fail_if_ratelimited: bool = True) -> List[SimpleUser]:
         """Fetches the last 1000 voters for your bot."""
         if not self.bot.is_ready():
@@ -164,7 +196,7 @@ class TopGG:
             stats["server_count"] = len(self.bot.guilds)
             if hasattr(self.bot, "shards") and self.bot.shards:
                 shards = []
-                for shard in self.bot.shards:
+                for shard in self.bot.shards.values():
                     shards.append(len([x for x in self.bot.guilds if x.shard_id == shard.id]))
                 stats["shards"] = shards
                 stats["shard_count"] = self.bot.shard_count
@@ -176,3 +208,13 @@ class TopGG:
             fail_if_timeout=fail_if_ratelimited
         )
         return stats["server_count"]
+
+    async def is_weekend(self) -> bool:
+        """Returns True or False, depending on if it's a "weekend".
+
+        If it's a weekend, votes count as double."""
+        data = await self._request(
+            "GET",
+            f"/weekend"
+        )
+        return data["is_weekend"]
